package io.flutter.plugins.firebaselivestreammlvision;

import androidx.annotation.NonNull;
import com.google.android.gms.tasks.OnFailureListener;
import com.google.android.gms.tasks.OnSuccessListener;
import com.google.firebase.ml.common.FirebaseMLException;
import com.google.firebase.ml.common.modeldownload.FirebaseModelDownloadConditions;
import com.google.firebase.ml.common.modeldownload.FirebaseModelManager;
import com.google.firebase.ml.common.modeldownload.FirebaseRemoteModel;
import com.google.firebase.ml.vision.FirebaseVision;
import com.google.firebase.ml.vision.common.FirebaseVisionImage;
import com.google.firebase.ml.vision.label.FirebaseVisionImageLabel;
import com.google.firebase.ml.vision.label.FirebaseVisionImageLabeler;
import com.google.firebase.ml.vision.label.FirebaseVisionOnDeviceAutoMLImageLabelerOptions;

import io.flutter.plugin.common.EventChannel;
import io.flutter.plugin.common.MethodChannel;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicBoolean;

class RemoteVisionEdgeDetector implements Detector {
  private FirebaseVisionImageLabeler labeler;

  RemoteVisionEdgeDetector(FirebaseVision vision, final Map<String, Object> options) {
    FirebaseRemoteModel remoteModel =
        FirebaseModelManager.getInstance().getNonBaseRemoteModel((String) options.get("dataset"));
    if (remoteModel == null) {
      FirebaseModelDownloadConditions conditions =
          new FirebaseModelDownloadConditions.Builder().build();
      remoteModel =
          new FirebaseRemoteModel.Builder((String) options.get("dataset"))
              .enableModelUpdates(true)
              .setInitialDownloadConditions(conditions)
              .setUpdatesDownloadConditions(conditions)
              .build();
      FirebaseModelManager.getInstance().registerRemoteModel(remoteModel);
      FirebaseModelManager.getInstance()
          .downloadRemoteModelIfNeeded(remoteModel)
          .addOnSuccessListener(
              new OnSuccessListener<Void>() {
                @Override
                public void onSuccess(Void success) {
                  try {
                    labeler =
                        FirebaseVision.getInstance()
                            .getOnDeviceAutoMLImageLabeler(parseOptions(options));
                  } catch (FirebaseMLException e) {
                    throw new IllegalArgumentException(e.getLocalizedMessage());
                  }
                }
              })
          .addOnFailureListener(
              new OnFailureListener() {
                @Override
                public void onFailure(@NonNull Exception e) {
                  throw new IllegalArgumentException(e.getLocalizedMessage());
                }
              });
      try {
        labeler = vision.getOnDeviceAutoMLImageLabeler(parseOptions(options));
      } catch (FirebaseMLException e) {
        throw new IllegalArgumentException(e.getLocalizedMessage());
      }
    } else {
      FirebaseModelManager.getInstance()
          .downloadRemoteModelIfNeeded(remoteModel)
          .addOnSuccessListener(
              new OnSuccessListener<Void>() {
                @Override
                public void onSuccess(Void success) {
                  try {
                    labeler =
                        FirebaseVision.getInstance()
                            .getOnDeviceAutoMLImageLabeler(parseOptions(options));
                  } catch (FirebaseMLException e) {
                    throw new IllegalArgumentException(e.getLocalizedMessage());
                  }
                }
              })
          .addOnFailureListener(
              new OnFailureListener() {
                @Override
                public void onFailure(@NonNull Exception e) {
                  throw new IllegalArgumentException(e.getLocalizedMessage());
                }
              });
    }
  }

  @Override
  public void handleDetection(final FirebaseVisionImage image, final EventChannel.EventSink result, final AtomicBoolean throttle) {
    labeler
        .processImage(image)
        .addOnSuccessListener(
            new OnSuccessListener<List<FirebaseVisionImageLabel>>() {
              @Override
              public void onSuccess(List<FirebaseVisionImageLabel> firebaseVisionLabels) {
                List<Map<String, Object>> labels = new ArrayList<>(firebaseVisionLabels.size());
                for (FirebaseVisionImageLabel label : firebaseVisionLabels) {
                  Map<String, Object> labelData = new HashMap<>();
                  labelData.put("confidence", (double) label.getConfidence());
                  labelData.put("text", label.getText());

                  labels.add(labelData);
                }

                  Map<String, Object> res = new HashMap<>();
                  res.put("eventType", "detection");
                  res.put("data", labels);
                  throttle.set(false);
                  result.success(res);
              }
            })
        .addOnFailureListener(
            new OnFailureListener() {
              @Override
              public void onFailure(@NonNull Exception e) {
                throttle.set(false);
                result.error("imageLabelerError", e.getLocalizedMessage(), null);
              }
            });
  }

  private FirebaseVisionOnDeviceAutoMLImageLabelerOptions parseOptions(
      Map<String, Object> optionsData) {
    float conf = (float) (double) optionsData.get("confidenceThreshold");
    return new FirebaseVisionOnDeviceAutoMLImageLabelerOptions.Builder()
        .setRemoteModelName((String) optionsData.get("dataset"))
        .setConfidenceThreshold(conf)
        .build();
  }

  @Override
  public void close() throws IOException {
    labeler.close();
  }
}
